@@ -194,7 +194,8 @@ It should return a string, with constraints:
     (id desc &optional time)
   "Example: \"[2025-02-21 Fri 14:39] <- [[id:ID][Node title]]\".
 ID and DESC are link id and description, TIME a Lisp time value."
-  (concat (org-node-time-stamp t t time)
+  (concat (format-time-string (org-time-stamp-format t t)
+                              (or time (current-time)))
           " <- "
           (org-link-make-string (concat "id:" id) desc)))
 
@@ -202,7 +203,8 @@ ID and DESC are link id and description, TIME a Lisp time value."
   "Example: \"- [2025-02-21 Fri 14:39] [[id:ID][Node title]]\".
 ID and DESC are link id and description, TIME a Lisp time value."
   (concat "- "
-          (org-node-time-stamp t t time)
+          (format-time-string (org-time-stamp-format t t)
+                              (or time (current-time)))
           " "
           (org-link-make-string (concat "id:" id) desc)))
 
@@ -377,10 +379,97 @@ Or if KIND is symbol `update-drawers', `del-drawers', `update-props', or
           (outline-next-heading))))))
 
 
-;;; Proactive updating
+;;; Save-hook to update only the changed parts of current buffer
 
-(defcustom org-node-backlink-lazy t
-  "Inhibit cleaning up backlinks until user edits affected entry.
+(defvar org-node-backlink--inhibit-flagging nil)
+(defun org-node-backlink--flag-buffer-modification (beg end _n-deleted-chars)
+  "Add text property `org-node-flag' to region between BEG and END.
+
+Designed for `after-change-functions', where this effectively flags
+all areas where text is added/changed/deleted.  Where text was
+purely deleted, this flags the preceding and succeeding char."
+  (unless org-node-backlink--inhibit-flagging
+    (with-silent-modifications
+      (if (= beg end)
+          (put-text-property (max (1- beg) (point-min))
+                             (min (1+ end) (point-max))
+                             'org-node-flag t)
+        (put-text-property beg end 'org-node-flag t)))))
+
+(defun org-node-backlink--fix-flagged-parts-of-buffer ()
+  "Fix backlinks around parts of buffer that have been modified.
+
+Look for areas flagged by
+`org-node-backlink--flag-buffer-modification' and run
+`org-node-backlink--fix-nearby' at each affected heading.
+
+For a huge file, this is much faster than using
+`org-node-backlink-fix-buffer' -- imagine a thousand
+headings but you have only done work under one of them."
+  (when (derived-mode-p 'org-mode)
+    (if (or (org-node-backlink--check-v2-misaligned-setting-p)
+            (org-node-backlink--check-osl-user-p))
+        (org-node-backlink-mode 0)
+      ;; Catch any error, because this runs at `before-save-hook' which MUST
+      ;; fail gracefully and let the user save anyway
+      (condition-case err
+          (save-excursion
+            (without-restriction
+              ;; Iterate over each change-region.  Algorithm borrowed from
+              ;; `ws-butler-map-changes', odd but elegant.  Worth knowing
+              ;; that if you tell Emacs to search for text that has a given
+              ;; text-property with a nil value, that's the same as searching
+              ;; for text without that property at all.  So if position START
+              ;; is in some unmodified area -- property `org-node-flag' is
+              ;; effectively valued at nil -- this way of calling
+              ;; `text-property-not-all' means search forward until it is t.
+              ;; Then calling it again has the opposite effect, searching
+              ;; until it is nil again.
+              (let ((start (point-min-marker))
+                    (end (make-marker))
+                    (case-fold-search t)
+                    prop)
+                (while (< start (point-max))
+                  (setq prop (get-text-property start 'org-node-flag))
+                  (set-marker end (or (text-property-not-all
+                                       start (point-max) 'org-node-flag prop)
+                                      (point-max)))
+                  (cl-assert (not (= start end)))
+                  (when prop
+                    (goto-char start)
+                    ;; START and END delineate an area where changes were
+                    ;; detected, but the area rarely envelops the current
+                    ;; tree's property drawer, likely placed long before
+                    ;; START, so search back for it
+                    (save-excursion
+                      (and (org-entry-get-with-inheritance "ID")
+                           (goto-char org-entry-property-inherited-from)
+                           (not (org-node--in-transclusion-p))
+                           (org-node-backlink--fix-nearby)))
+                    ;; ...and if the change-area is massive, spanning multiple
+                    ;; subtrees (like after a big yank), update each subtree
+                    ;; within
+                    (while (and (< (point) end)
+                                (re-search-forward "^[\t\s]*:id: +" end t))
+                      (unless (org-node--in-transclusion-p)
+                        (org-node-backlink--fix-nearby)))
+                    (remove-text-properties start end 'org-node-flag))
+                  ;; This change-area dealt with, move on
+                  (set-marker start (marker-position end)))
+                (set-marker start nil)
+                (set-marker end nil))))
+        (( error )
+         (unless (remove-text-properties 1 (point-max) 'org-node-flag)
+           (message "org-node: Did not remove org-node-flag text property"))
+         (message "org-node: Updating backlinks ran into an issue: %S" err))))))
+
+
+;;; Proactive updating (broken & disabled for now)
+
+(defcustom org-node-backlink-lazy nil
+  "NO EFFECT: Broken since org-node v3.
+
+Inhibit cleaning up backlinks until user edits affected entry.
 
 Background: Regardless of this value, links inserted via most commands
 will insert a backlink in real time, so long as
@@ -409,66 +498,68 @@ To force an update at any time, use one of these commands:
   :type 'boolean
   :package-version '(org-node . "2.0.0"))
 
+;; XXX Broken 2025-05-22 (visits too many entries => saving big file is super slow)
+;;     No-op for now
 (defun org-node-backlink--maybe-fix-proactively (_)
   "Designed for `org-mem-post-targeted-scan-functions'."
-  (unless org-node-backlink-lazy
+  (unless t ;; org-node-backlink-lazy
     (let (affected-targets)
-      (dolist (target (cl-union (hash-table-keys org-mem--target<>old-links)
-                                (hash-table-keys org-mem--target<>links)))
-        (let ((entry (or (org-mem-entry-by-id target)
-                         (org-mem-entry-by-roam-ref target))))
-          ;; Entry being targeted must have an ID, or we can't use
-          ;; `org-find-property' later.
-          (when (and entry (org-mem-entry-id entry))
-            (let ((links (gethash target org-mem--target<>links))
-                  (old-links (gethash target org-mem--target<>old-links)))
-              (when (or (null links) ;; Gone: no longer any links with this target
-                        (null old-links) ;; New: this target never been linked before
-                        (not
-                         ;; The set of "source entries" where the links were
-                         ;; found, do they have same IDs they did before?
-                         ;; If not, then something changed.
-                         ;; (Either there's a new source or a source stopped
-                         ;; linking to this target.)
-                         (seq-set-equal-p
-                          (seq-keep #'org-mem-link-nearby-id old-links)
-                          (seq-keep #'org-mem-link-nearby-id links))))
-                ;; Something changed.  So we'll visit the targeted entry to
-                ;; re-print backlinks.
-                ;; Alist `affected-targets' looks like:
-                ;;   ((file1 . (id1 id2 id3 ...))
-                ;;    (file2 . (...))
-                ;;    (file3 . (...)))
-                (push (org-mem-entry-id entry)
-                      (alist-get (org-mem-entry-file-truename entry)
-                                 affected-targets
-                                 nil
-                                 nil
-                                 #'equal)))))))
+      (cl-loop
+       for target being each hash-key of org-mem-updater--id-or-ref-target<>old-links
+       using (hash-values old-links)
+       as entry = (or (org-mem-entry-by-id target)
+                      (org-mem-entry-by-roam-ref target))
+       ;; Entry must have ID or we can't use `org-find-property' later.
+       when (and entry (org-mem-entry-id entry))
+       when (not (seq-set-equal-p
+                  (seq-keep #'org-mem-link-nearby-id old-links)
+                  (seq-keep #'org-mem-link-nearby-id
+                            (append (org-mem-roam-reflinks-to-entry entry)
+                                    (org-mem-id-links-to-entry entry)))))
+       ;; Something changed in the set of links targeting this entry.  So we'll
+       ;; visit the entry to re-print backlinks.
+       ;; Alist `affected-targets' looks like:
+       ;;   ((file1 . (id1 id2 id3 ...))
+       ;;    (file2 . (...))
+       ;;    (file3 . (...)))
+       do
+       (push (org-mem-entry-id entry)
+             (alist-get (org-mem-entry-file-truename entry)
+                        affected-targets
+                        () () #'equal)))
+      (cl-loop for target in org-mem-updater--new-id-or-ref-targets
+               do (cl-loop
+                   for link in (gethash target org-mem--target<>links)
+                   as id = (org-mem-link-nearby-id link)
+                   when id do
+                   (push id (alist-get (org-mem-link-file-truename link)
+                                       affected-targets
+                                       () () #'equal))))
+
       (cl-loop
        for (file . ids) in affected-targets
-       do (cond
-           ((not (file-readable-p file))
-            (message "Cannot edit backlinks in unreadable file: %s" file))
-           ((not (file-writable-p file))
-            (message "Cannot edit backlinks in unwritable file: %s" file))
-           ((org-node--with-quick-file-buffer file
-              :about-to-do "About to fix backlinks"
-              (org-node--assert-transclusion-safe)
-              (let ((user-is-editing (buffer-modified-p)))
-                (dolist (id (delete-dups ids))
-                  (if-let* ((pos (and id (org-find-property "ID" id))))
-                      (progn (goto-char pos)
-                             (org-node-backlink--fix-nearby))
-                    (error "Could not find ID %s in file %s" id file)))
-                (unless user-is-editing
-                  ;; Normally, `org-node--with-quick-file-buffer' only saves buffers
-                  ;; it had to open anew.  Let's save even if it was open previously.
-                  (let ((before-save-hook nil)
-                        (after-save-hook nil)
-                        (save-silently t)
-                        (inhibit-message t))
-                    (save-buffer)))))))))))
+       if (not (file-readable-p file))
+       do (message "Cannot edit backlinks in unreadable file: %s" file)
+       else if (not (file-writable-p file))
+       do (message "Cannot edit backlinks in unwritable file: %s" file)
+       else do
+       (org-node--with-quick-file-buffer file
+         :about-to-do "About to fix backlinks"
+         (org-node--assert-transclusion-safe)
+         (let ((user-is-editing (buffer-modified-p)))
+           (dolist (id (delete-dups ids))
+             (if-let* ((pos (and id (org-find-property "ID" id))))
+                 (progn (goto-char pos)
+                        (org-node-backlink--fix-nearby))
+               (error "Could not find ID %s in file %s" id file)))
+           (unless user-is-editing
+             ;; Normally, `org-node--with-quick-file-buffer' only saves buffers
+             ;; it had to open anew.  Let's save even if it was open previously.
+             (let ((before-save-hook nil)
+                   (after-save-hook nil)
+                   (save-silently t)
+                   (inhibit-message t))
+               (save-buffer)))))))))
 
 
 ;;; Subroutine: "Fix nearby"
@@ -540,7 +631,7 @@ If REMOVE non-nil, remove it instead."
                            ;; Shouldn't be necessary if tables are correct, but
                            ;; don't assume `org-mem-updater-mode' is flawless
                            (seq-filter #'org-mem-entry-by-id))))
-           (org-node--inhibit-flagging t))
+           (org-node-backlink--inhibit-flagging t))
       (if (null origin-ids)
           (org-node--delete-drawer "BACKLINKS")
         (save-excursion
@@ -626,13 +717,11 @@ If REMOVE non-nil, remove it instead."
          target-id target-file)
     ;; In a link such as [[id:abc1234]], TYPE is "id" and PATH is "abc1234".
     (when (and type path)
-      ;; REVIEW 2025-11-08: Testing with this commented-out.
-      ;;
       ;; Ensure `org-mem--roam-ref<>id' has recent goods, and that
-      ;; `org-node-backlink--fix-nearby' will not
+      ;; `org-node-backlink--fix-flagged-parts-of-buffer' will not
       ;; later remove the backlink we're adding
-      ;; (org-mem-updater-ensure-id-node-at-point-known)
-      ;; (org-mem-updater-ensure-link-at-point-known)
+      (org-mem-updater-ensure-id-node-at-point-known)
+      (org-mem-updater-ensure-link-at-point-known)
       (if (equal "id" type)
           ;; A classic backlink
           (progn
@@ -684,7 +773,7 @@ If REMOVE non-nil, remove it instead."
   (let ((current-backlinks-value (org-entry-get nil "BACKLINKS"))
         (new-link (org-link-make-string (concat "id:" id) title))
         new-value
-        (org-node--inhibit-flagging t))
+        (org-node-backlink--inhibit-flagging t))
     (when (and current-backlinks-value
                (string-search "\f" current-backlinks-value))
       (error "Form-feed character in BACKLINKS property near %d in %s"
@@ -730,7 +819,7 @@ Designed for use by `org-node-backlink--add-in-target'."
       (org-node-backlink-mode 0)
     (save-excursion
       (save-restriction
-        (let ((org-node--inhibit-flagging t))
+        (let ((org-node-backlink--inhibit-flagging t))
           (org-node-narrow-to-drawer-create
            "BACKLINKS" org-node-backlink-drawer-positioner)
           (unless (search-forward (concat "[[id:" id) nil t) ;; Already has it
@@ -761,18 +850,36 @@ See Info node `(org-node)'."
   :global t
   (if org-node-backlink-mode
       (progn
+        (add-hook 'org-mode-hook                        #'org-node-backlink--local-mode)
         (add-hook 'org-mem-post-targeted-scan-functions #'org-node-backlink--maybe-fix-proactively)
         (add-hook 'org-node-relocation-hook             #'org-node-backlink--fix-nearby)
-        (add-hook 'org-node-modification-hook           #'org-node-backlink--fix-nearby)
         (add-hook 'org-roam-post-node-insert-hook       #'org-node-backlink--add-in-target)
         (add-hook 'org-node-insert-link-hook            #'org-node-backlink--add-in-target)
-        (advice-add 'org-insert-link :after             #'org-node-backlink--add-in-target))
+        (advice-add 'org-insert-link :after             #'org-node-backlink--add-in-target)
+        (dolist (buf (buffer-list))
+          (with-current-buffer buf
+            (when (derived-mode-p 'org-mode)
+              (org-node-backlink--local-mode)))))
+    (remove-hook 'org-mode-hook                        #'org-node-backlink--local-mode)
     (remove-hook 'org-mem-post-targeted-scan-functions #'org-node-backlink--maybe-fix-proactively)
     (remove-hook 'org-node-relocation-hook             #'org-node-backlink--fix-nearby)
-    (remove-hook 'org-node-modification-hook           #'org-node-backlink--fix-nearby)
     (remove-hook 'org-roam-post-node-insert-hook       #'org-node-backlink--add-in-target)
     (remove-hook 'org-node-insert-link-hook            #'org-node-backlink--add-in-target)
-    (advice-remove 'org-insert-link                    #'org-node-backlink--add-in-target)))
+    (advice-remove 'org-insert-link                    #'org-node-backlink--add-in-target)
+    (dolist (buf (buffer-list))
+      (with-current-buffer buf
+        (org-node-backlink--local-mode 0)))))
+
+(define-minor-mode org-node-backlink--local-mode
+  "Buffer-local part of `org-node-backlink-mode'.
+Not meant to be toggled on its own."
+  :interactive nil
+  (if org-node-backlink--local-mode
+      (progn
+        (add-hook 'after-change-functions #'org-node-backlink--flag-buffer-modification nil t)
+        (add-hook 'before-save-hook       #'org-node-backlink--fix-flagged-parts-of-buffer nil t))
+    (remove-hook 'after-change-functions  #'org-node-backlink--flag-buffer-modification t)
+    (remove-hook 'before-save-hook        #'org-node-backlink--fix-flagged-parts-of-buffer t)))
 
 (let (warned-once)
   (defun org-node-backlinks-mode (&rest args)
