@@ -1,13 +1,13 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "org-node" "3.17.2"
+(define-package "org-node" "20251007.2201"
   "Fast org-roam replacement."
   '((emacs         "29.1")
-    (llama         "1.0")
-    (magit-section "4.3.0")
-    (org-mem       "0.32.0"))
+    (llama         "0.5.0")
+    (org-mem       "0.22.0")
+    (magit-section "4.3.0"))
   :url "https://github.com/meedstrom/org-node"
-  :commit "0ab480012fe0c40d2894546cdef909495d582a0d"
-  :revdesc "3.17.2-0-g0ab480012fe0"
+  :commit "57c57017b42167401d5eed8b62ad3e11f7714c7b"
+  :revdesc "57c57017b421"
   :keywords '("org" "hypermedia")
   :authors '(("Martin Edström" . "meedstrom91@gmail.com"))
   :maintainers '(("Martin Edström" . "meedstrom91@gmail.com")))
