@@ -1,6 +1,6 @@
 ;;; org-node-seq.el --- Experimental way to define node sequences -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2024-2026 Martin Edström
+;; Copyright (C) 2024-2025 Martin Edström
 ;;
 ;; This program is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -156,7 +156,7 @@ YYYY-MM-DD format, e.g. \"2024-01-31.org\"."
                            (sortstr (file-name-base path)))
                       (cons sortstr path))))
     :whereami (lambda ()
-                (let ((file-name (org-mem-file-known-p buffer-file-name)))
+                (let ((file-name (org-mem--truename-maybe buffer-file-name)))
                   (when (string-prefix-p ,dir file-name)
                     (file-name-base file-name))))
     :prompter (lambda (key)
@@ -289,7 +289,7 @@ Jump/Next/Previous/Capture actions.
 
 NAME describes the sequence, in one or a few words.
 
-CLASSIFIER is a single-argument function taking an `org-mem-entry' object
+CLASSIFIER is a single-argument function taking an `org-node' object
 and should return a list or cons cell if a sequence-item was found,
 otherwise nil.
 
@@ -560,6 +560,9 @@ This permits \\[org-node-seq-dispatch] to work."
       (progn
         (org-node-cache-ensure t)
         (org-node-seq--reset)
+        ;; FIXME: A new node (cached w `org-mem-updater-ensure-id-node-at-point-known')
+        ;;        eventually disappears from cache if its buffer is never
+        ;;        saved, and then the node seq stops working
         (add-hook 'org-node-creation-hook        #'org-node-seq--add-item)
         (add-hook 'org-mem-post-full-scan-functions #'org-node-seq--reset 50)
         ;; Put ourselves in front of org-roam-dailies unhygienic hook.
